local L, _TRANS, CreateFrame, hooksecurefunc, getreplacehook = unpack((select(2, ...)).locals)
function Nx.HUD:Init()
	Nx.HUD.TexNames = { "", "Chip", "Gloss", "Glow", "Neon" }
	Nx.HUD:Open()
end
function Nx.HUD:Open()
	if not self.Created then
		self:Create()
		self.Created = true
	end
	local inst = self
	inst.Win:Show()
end
function Nx.HUD:Create()
	local inst = self
	local gopts = Nx.GetGlobalOpts()
	inst.GOpts = gopts
	inst.ETADelay = 0
	
	Nx.Window:SetCreateFade (1, .15)
	local win = Nx.Window:Create ("NxHUD", nil, nil, nil, 2, 1, nil, true)
	inst.Win = win
	win:SetTitleJustify ("CENTER", 1)
	win:SetTitleJustify ("CENTER", 2)
	win:SetBGAlpha (0, 1)
	win:InitLayoutData (nil, 999999, -.17, 1, 1)
	win.Frm:SetToplevel (true)
	
	local f = CreateFrame ("Frame", nil, win.Frm)
	inst.Frm = f
	f.NxInst = inst
	f:EnableMouse (false)
	local t = f:CreateTexture()
	t:SetAllPoints (f)
	f.texture = t
	
	local but = CreateFrame ("Button", nil, UIParent, "SecureUnitButtonTemplate")
	inst.But = but
	but:SetAttribute ("type", "target")
	but:SetAttribute ("unit", "player")
	but:RegisterForClicks ("LeftButtonDown", "RightButtonDown")
	local t = but:CreateTexture()
	t:SetAllPoints (but)
	t:SetTexture ("Interface\\AddOns\\Carbonite\\Gfx\\Map\\IconCircle")
	but.texture = t
	but:SetWidth (10)
	but:SetHeight (10)
	
	self:UpdateOptions()
end
function Nx.HUDGetTracking()
	local map = Nx.Map:GetMap (1)
	return map.TrackDir, map.TrackDistYd, map.TrackName
end
function Nx.HUD:SetFade (fade)
end
function Nx.HUD:Show (show)
	self.Win:Show (show)
end
function Nx.HUD:UpdateOptions()
	local win = self.Win
	if not Nx.Free then
		local lock = win:IsLocked()
		win:SetBGAlpha (0, lock and 0 or 1)
	end
	local gopts = self.GOpts
	local name = gopts["HUDAGfx"]
	self.Frm.texture:SetTexture ("Interface\\AddOns\\Carbonite\\Gfx\\Map\\HUDArrow" .. name)
	local f = self.Frm
	f:SetPoint ("CENTER", gopts["HUDAXO"], -win.TitleH / 2 - 32 - gopts["HUDAYO"])
	local wh = gopts["HUDASize"]
	f:SetWidth (wh)
	f:SetHeight (wh)
	if not InCombatLockdown() then
		local f = self.But
		f:SetWidth (wh)
		f:SetHeight (wh)
		f:Hide()
	end
	self.ButR, self.ButG, self.ButB, self.ButA = Nx.Util_num2rgba (gopts["HUDTButColor"])
	self.ButCR, self.ButCG, self.ButCB, self.ButCA = Nx.Util_num2rgba (gopts["HUDTButCombatColor"])
end
function Nx.HUD:Update (map)
	local win = self.Win
	local gopts = self.GOpts
	local opts = Nx:GetHUDOpts()
	local noLockDown = not InCombatLockdown()
	if map.TrackDir and not gopts["HUDHide"] and not (Nx.InBG and gopts["HUDHideInBG"]) then
		local frm = self.Frm
		local but = self.But
		local wfrm = win.Frm
		if not wfrm:IsVisible() then	
			if not win:IsCombatHidden() then
				win:Show()
			end
		end
		local dist = map.TrackDistYd
		local dir = (map.TrackDir - map.PlyrDir) % 360
		if dist < 1 then
			dir = 0
		end
		local dirDist = dir <= 180 and dir or 360 - dir
		local str = _TRANS(map.TrackName) or ""
		win:SetTitle (str)
		if map.TrackPlayer and noLockDown then
			but:SetAttribute ("unit1", map.TrackPlayer)
			but:SetAttribute ("shift-unit1", map.TrackPlayer .. "-target")
			but:SetAttribute ("unit2", map.TrackPlayer .. "-target")
		end
		local col = dirDist < 5 and "|cffa0a0ff" or ""
		local str = format (L"%s%d yds", col, dist)
		if gopts["HUDShowDir"] then
			local fmt = dirDist < 1 and L" %.1f deg" or L" %d deg"
			str = str .. format (fmt, dirDist)
		end
		if map.PlyrSpeed > .1 then
			self.ETADelay = self.ETADelay - 1
			if self.ETADelay <= 0 then
				self.ETADelay = 10
				local eta = map.TrackETA or dist / map.PlyrSpeed
				if eta < 60 then
					self.ETAStr = format (L"|cffdfffdf %.0f secs", eta)
				else
					self.ETAStr = format (L"|cffdfdfdf %.1f mins", eta / 60)
				end
			end
			str = str .. self.ETAStr
		else
			self.ETADelay = 3		
			self.ETAStr = ""
		end
		win:SetTitle (str, 2)
		local atPt, relTo, relPt, x, y = wfrm:GetPoint()
		local w, h = win:GetSize()
		local tw = win:GetTitleTextWidth() + 2
		local d = (tw - w) / 2
		if strfind (atPt, "LEFT") then
			x = x - d
		elseif strfind (atPt, "RIGHT") then
			x = x + d
		end
		wfrm:ClearAllPoints()
		wfrm:SetPoint (atPt, x, y)
		win:SetSize (tw, 0, true)
		if gopts["HUDTBut"] and not win:IsCombatHidden() then
			if noLockDown then
				but:SetPoint ("TOPLEFT", UIParent, "BOTTOMLEFT", frm:GetLeft(), frm:GetTop())
				but:SetScale (wfrm:GetScale())
				but:Show()
				but.texture:SetVertexColor (self.ButR, self.ButG, self.ButB, self.ButA)
			else
				but.texture:SetVertexColor (self.ButCR, self.ButCG, self.ButCB, self.ButCA)
			end
		end
		local texX1 = -.5
		local texX2 = .5
		local texY1 = -.5
		local texY2 = .5
		local co = cos (dir)
		local si = sin (dir)
		t1x = texX1 * co + texY1 * si + .5
		t1y = texX1 * -si + texY1 * co + .5
		t2x = texX1 * co + texY2 * si + .5
		t2y = texX1 * -si + texY2 * co + .5
		t3x = texX2 * co + texY1 * si + .5
		t3y = texX2 * -si + texY1 * co + .5
		t4x = texX2 * co + texY2 * si + .5
		t4y = texX2 * -si + texY2 * co + .5
		local tex = frm.texture
		tex:SetTexCoord (t1x, t1y, t2x, t2y, t3x, t3y, t4x, t4y)
		if dirDist < 5 then
			if dist < 1 then
				tex:SetVertexColor (.2, 1, .2, .4)
				tex:SetBlendMode ("BLEND")
			else
				tex:SetVertexColor (.7, .7, 1, 1)
				tex:SetBlendMode ("ADD")
			end
		else
			tex:SetVertexColor (1, 1, .5, .9)
			tex:SetBlendMode ("BLEND")
		end
	else
		win:Show (false)
		if noLockDown then
			self.But:Hide()
		end
	end
end
