local L, _TRANS, CreateFrame, hooksecurefunc, getreplacehook = unpack((select(2, ...)).locals)
function Nx.Timer:Init()
	self.Data = {}
	self:ProfilerInit()
end
function Nx.Timer:Start (name, time, user, func)
	if not self.Data[name] then
		self.Data[name] = {}
	end
	local tm = self.Data[name]
	tm.T = time
	tm.U = user
	tm.F = func
	return tm
end
function Nx.Timer:Stop (name)
	self.Data[name] = nil
end
function Nx.Timer:SetFunc (name, func)
	if self.Data[name] then
		self.Data[name].F = func
	end
end
function Nx.Timer:IsActive (name)
	return self.Data[name]
end
function Nx.Timer:Time (name)
	if self.Data[name] then
		return self.Data[name].T
	end
end
function Nx.Timer:OnUpdate (elapsed)
	elapsed = min (elapsed, .5)
	for name, tm in pairs (self.Data) do
		tm.T = tm.T - elapsed
		if tm.T <= 0 then
			if tm.F then
                
				tm.T = tm.F (tm.U, name, tm)
			end
			if not tm.T then
				self.Data[name] = nil
			end
		end
	end
	self:ProfilerOnUpdate()
end
function Nx.Timer:Fire (name)
	local tm = self.Data[name]
	if tm then
		if tm.F then
			tm.T = tm.F (tm.U, name, tm)
		end
		if not tm.T then
			self.Data[name] = nil
		end
	end
end
function Nx.Timer.Win:Open()
	if Nx.Free then
		return
	end
	local win = self.Win
	if win then
		if win:IsShown() then
			win:Show (false)
		else
			win:Show()
		end
		return
	end
end
function Nx.Timer:ProfilerInit()
	self.Profiles = {}
	self.RunTime = GetTime()
end
function Nx.Timer:ProfilerOnUpdate()
end
function Nx.Timer:ProfilerStart (name, reset)
	local pro = self.Profiles[name]
	if not pro then
		pro = {}
		self.Profiles[name] = pro
		tinsert (self.Profiles, pro)
		pro.Name = name
		pro.Time = 0
		pro.TimeLast = 0
		pro.Cnt = 0
		pro.Reset = reset
	end
	pro.Start = GetTime()
	pro.Cnt = pro.Cnt + 1
end
function Nx.Timer:ProfilerEnd (name)
	local pro = self.Profiles[name]
	assert (pro)
	pro.TimeLast = GetTime() - pro.Start
	pro.Time = pro.Time + pro.TimeLast
end
function Nx.Timer:ProfilerGetLastTime (name)
	local pro = self.Profiles[name]
	return pro and pro.TimeLast or 0
end
function Nx.Timer:ProfilerDump()
	sort (self.Profiles, function (a, b) return a.Name < b.Name end)
	Nx.prt ("Profiler: FPS %.0f", GetFramerate())
	for _, pro in ipairs (self.Profiles) do
		Nx.prt (" %s %.4f %.4f avrg, #%s, %.3f tot", pro.Name, pro.TimeLast, pro.Time / pro.Cnt, pro.Cnt, pro.Time)
	end
	Nx.prt ("Run time %.0f", GetTime() - self.RunTime)
end
